/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import java.io.File;


/**
 * <p>
 * This class is a base class representing a database server to be used while running tests. The initializing data is
 * stored inside a JAR (Java Archive) file. It is up to the subclass implementation on what data should be stored. For
 * example, a sql script can be retrieved and executed to create the tables and insert some testing data.
 * </p>
 *
 * <p>
 * In the build file, this class represents the child element under the ServerElement. The name of the element is
 * supplied to the constructor. It is up to the subclass implementation on how to use it. The element supports the
 * following attributes, in addition to the attributes supported by the base class:
 *
 * <ul>
 * <li>
 * <b>jarfile</b> - the Java Archive that contains the data to initialize the database server with. Can be absolute or
 * relative (to the project's basedir). Subclass should specify if this attribute is required.
 * </li>
 * </ul>
 *
 * The nested elements supported by the base class are inherited as well.
 * </p>
 *
 * <p>
 * For those methods called by the Ant framework, such as attribute setters, there is no check for arguments against
 * null. The Ant framework makes sure that the arguments will not be null.
 * </p>
 *
 * <p>
 * This class is not thread-safe. However the methods will be called one by one by the Ant framework according to the
 * life cycle of ant task. So thread-safety is not an issue here.
 * </p>
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public abstract class DataServer extends Server {
    /**
     * <p>
     * The "jarfile" attribute specifies the Java Archive that contains the data to initialize the database server
     * with. Subclass should specify if this attribute is required. It is initially null, and set later by the
     * setJarFile() method.
     * </p>
     */
    private File jarFile = null;

    /**
     * <p>
     * Creates a new instance of this class with the specified type. The type is the name of the element represented by
     * this class. This constructor simply calls the super constructor.
     * </p>
     *
     * @param type the type of the server.
     *
     * @throws IllegalArgumentException if type is null or empty string.
     * @throws BuildException if the type of the server is unrecognized.
     */
    protected DataServer(String type) {
        super(type);
    }

    /**
     * <p>
     * Sets the value of the "jarfile" attribute. The "jarfile" attribute specifies the Java Archive that contains the
     * data to initialize the database server with.
     * </p>
     *
     * @param jarFile the value of the "jarfile" attribute.
     */
    public void setJarFile(File jarFile) {
        this.jarFile = jarFile;
    }

    /**
     * <p>
     * Returns the value of the "jarfile" attribute. The "jarfile" attribute specifies the Java Archive that contains
     * the data to initialize the database server with.
     * </p>
     *
     * @return the value of the "jarfile" attribute, or null if none is specified.
     */
    public File getJarFile() {
        return jarFile;
    }
}
