/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import org.apache.tools.ant.BuildException;

import java.io.File;


/**
 * <p>
 * This class is a base class representing an actual server to be used while running tests. A server can be optionally
 * started up and initialized with data before running the tests, and in the same way, cleaned up for the data and
 * shut down after running the tests.
 * </p>
 *
 * <p>
 * In the build file, this class represents the child element under the ServerElement. The name of the element is
 * supplied to the constructor. It is up to the subclass implementation on how to use it, for instance,
 * DefaultWebApplicationServer relies on this name to create the Container using the ContainerFactory (in the Cactus
 * environment). The element supports the following attributes:
 *
 * <ul>
 * <li>
 * <b>dir</b> (required) - the installation directory of the server. Can be absolute or relative (to the project's
 * basedir).
 * </li>
 * <li>
 * <b>port</b> (optional) - the TCP port that the server should listen to. Must be positive. Default value can be
 * defined by the derived classes.
 * </li>
 * <li>
 * <b>server</b> (optional) - the host name or IP address of the machine running the server. Default is "localhost".
 * </li>
 * <li>
 * <b>startserver</b> (optional) - a boolean indicating whether the server should be started up (and shut down) before
 * (and after) running the tests. Default is true.
 * </li>
 * <li>
 * <b>initserver</b> (optional) - a boolean indicating whether the server should be initialized with data (and cleaned
 * up for the data) before (and after) running the tests. Default is true.
 * </li>
 * </ul>
 *
 * Additionally, it can contain a <b>credentials</b> element (optional) specifying the authentication properties (as
 * sub-elements) for accessing the server, such as user name and password. Again, the way to use all these attributes
 * is up to the subclass implementation.
 * </p>
 *
 * <p>
 * Note that the constructor of the Server implementation must accept one String argument specifying the type of the
 * server, in order for the class to be instantiated by reflection.
 * </p>
 *
 * <p>
 * For those methods called by the Ant framework, such as attribute setters, there is no check for arguments against
 * null. The Ant framework makes sure that the arguments will not be null.
 * </p>
 *
 * <p>
 * This class is not thread-safe. However the methods will be called one by one by the Ant framework according to the
 * life cycle of ant task. So thread-safety is not an issue here.
 * </p>
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public abstract class Server {
    /**
     * <p>
     * This constant represents the user name for authentication to the server. It should be used as the name of the
     * sub-element under the "credentials" element.
     * </p>
     */
    public static final String USER_NAME = "user_name";

    /**
     * <p>
     * This constant represents the password for authentication to the server. It should be used as the name of the
     * sub-element under the "credentials" element.
     * </p>
     */
    public static final String PASSWORD = "password";

    /**
     * <p>
     * The "dir" attribute represents the installation directory of the server. This attribute is required. It is
     * initially null, and set later by the setDir() method.
     * </p>
     */
    private File dir = null;

    /**
     * <p>
     * The "port" attribute represents the TCP port that the server should listen to. This attribute is optional. The
     * default value can be defined by the derived classes. It is initially -1, and set later by the setPort() method.
     * </p>
     */
    private int port = -1;

    /**
     * <p>
     * The "server" attribute represents the host name or IP address of the machine running the server. This attribute
     * is optional. It is initially set to "localhost", and set later by the setServer() method.
     * </p>
     */
    private String server = "localhost";

    /**
     * <p>
     * The "startserver" attribute specifies whether the server should be started up (and shut down) before (and after)
     * running the tests. This attribute is optional. It is initially true, and set later by the setStartServer()
     * method.
     * </p>
     */
    private boolean startServer = true;

    /**
     * <p>
     * Represents an encapsulation of the information about the ant task. It is initially null, and set later by the
     * setAntTaskInfo() method.
     * </p>
     */
    private AntTaskInfo antTaskInfo = null;

    /**
     * <p>
     * The "initserver" attribute specifies whether the server should be initialized with data (and cleaned up for the
     * data) before (and after) running the tests. This attribute is optional. It is initially true, and set later by
     * the setInitServer() method.
     * </p>
     */
    private boolean initServer = true;

    /**
     * <p>
     * The nested "credentials" element, which represents the authentication properties for accessing the server, such
     * as user name and password. This element is optional. It is initially null, and set later by the
     * createCredentials() method.
     * </p>
     */
    private CredentialsElement credentials = null;

    /**
     * <p>
     * Creates a new instance of this class with the specified type. The type is the name of the element represented by
     * this class. This constructor does nothing except argument check.
     * </p>
     *
     * @param type the type of the server.
     *
     * @throws IllegalArgumentException if type is null or empty string.
     * @throws BuildException if the type of the server is unrecognized.
     */
    protected Server(String type) {
        if (type == null) {
            throw new IllegalArgumentException("type is null");
        }
        if (type.trim().length() == 0) {
            throw new IllegalArgumentException("type is empty string");
        }
    }

    /**
     * <p>
     * Starts the server. This method should be called before running the tests. Subclasses must implement this method
     * to perform the actual task of starting up the server, possibly taking the "startserver" attribute into account.
     * </p>
     *
     * @throws BuildException if it fails to start up the server.
     */
    public abstract void startUp();

    /**
     * <p>
     * Stops the server. This method should be called after running the tests. Subclasses must implement this method to
     * perform the actual task of stopping the server, possibly taking the "startserver" attribute into account.
     * </p>
     *
     * @throws BuildException if it fails to stop the server.
     */
    public abstract void shutDown();

    /**
     * <p>
     * Initializes the server with data. This method should be called before running the tests. Subclasses must
     * implement this method to perform the actual task of initializing the server with data, possibly taking the
     * "initserver" attribute into account.
     * </p>
     *
     * @throws BuildException if it fails to initialize the server with data.
     */
    public abstract void initData();

    /**
     * <p>
     * Cleans up the data for the server initialized by the initData() method. This method should be called after
     * running the tests. Subclasses must implement this method to perform the actual task of cleaning up the data,
     * possibly taking the "initserver" attribute into account.
     * </p>
     *
     * @throws BuildException if it fails to clean up the data.
     */
    public abstract void cleanUpData();

    /**
     * <p>
     * Sets the value of the "dir" attribute. The "dir" attribute represents the installation directory of the server.
     * </p>
     *
     * @param dir the value of the "dir" attribute.
     */
    public void setDir(File dir) {
        this.dir = dir;
    }

    /**
     * <p>
     * Returns the value of the "dir" attribute. The "dir" attribute represents the installation directory of the
     * server.
     * </p>
     *
     * @return the value of the "dir" attribute, or null if none is specified.
     */
    public File getDir() {
        return dir;
    }

    /**
     * <p>
     * Sets the value of the "port" attribute. The "port" attribute represents the TCP port that the server should
     * listen to.
     * </p>
     *
     * @param port the value of the "port" attribute.
     *
     * @throws IllegalArgumentException if port is non-positive.
     */
    public void setPort(int port) {
        if (port <= 0) {
            throw new IllegalArgumentException("port is not positive");
        }
        this.port = port;
    }

    /**
     * <p>
     * Returns the value of the "port" attribute. The "port" attribute represents the TCP port that the server should
     * listen to.
     * </p>
     *
     * @return the value of the "port" attribute, or -1 if none is specified.
     */
    public int getPort() {
        return port;
    }

    /**
     * <p>
     * Sets the value of the "server" attribute. The "server" attribute represents the host name or IP address of the
     * machine running the server.
     * </p>
     *
     * @param server the value of the "server" attribute.
     *
     * @throws IllegalArgumentException if server is empty string
     */
    public void setServer(String server) {
        if (server.trim().length() == 0) {
            throw new IllegalArgumentException("server is empty string");
        }
        this.server = server;
    }

    /**
     * <p>
     * Returns the value of the "server" attribute. The "server" attribute represents the host name or IP address of
     * the machine running the server.
     * </p>
     *
     * @return the value of the "server" attribute.
     */
    public String getServer() {
        return server;
    }

    /**
     * <p>
     * Sets the value of the "startserver" attribute. The "startserver" attribute specifies whether the server should
     * be started up (and shut down) before (and after) running the tests.
     * </p>
     *
     * @param startServer the value of the "startserver" attribute.
     */
    public void setStartServer(boolean startServer) {
        this.startServer = startServer;
    }

    /**
     * <p>
     * Returns the value of the "startserver" attribute. The "startserver" attribute specifies whether the server
     * should be started up (and shut down) before (and after) running the tests.
     * </p>
     *
     * @return the value of the "startserver" attribute.
     */
    public boolean getStartServer() {
        return startServer;
    }

    /**
     * <p>
     * Sets the information about the ant task being run. Note that the argument is checked here since it is not an
     * attribute set by the Ant framework.
     * </p>
     *
     * @param antTaskInfo the information about the ant task.
     *
     * @throws IllegalArgumentException if antTaskInfo is null
     */
    public void setAntTaskInfo(AntTaskInfo antTaskInfo) {
        if (antTaskInfo == null) {
            throw new IllegalArgumentException("antTaskInfo is null");
        }
        this.antTaskInfo = antTaskInfo;
    }

    /**
     * <p>
     * Returns the information about the ant task being run.
     * </p>
     *
     * @return the information about the ant task being run.
     */
    public AntTaskInfo getAntTaskInfo() {
        return antTaskInfo;
    }

    /**
     * <p>
     * Sets the value of the "initserver" attribute. The "initserver" attribute specifies whether the server should be
     * initialized with data (and cleaned up for the data) before (and after) running the tests.
     * </p>
     *
     * @param initServer the value of the "initserver" attribute.
     */
    public void setInitServer(boolean initServer) {
        this.initServer = initServer;
    }

    /**
     * <p>
     * Returns the value of the "initserver" attribute. The "initserver" attribute specifies whether the server should
     * be initialized with data (and cleaned up for the data) before (and after) running the tests.
     * </p>
     *
     * @return the value of the "initserver" attribute.
     */
    public boolean getInitServer() {
        return initServer;
    }

    /**
     * <p>
     * Creates a nested "credentials" element. The "credentials" element specifies the authentication properties for
     * accessing the server.
     * </p>
     *
     * @return the nested "credentials" element.
     *
     * @throws BuildException if there are more than one nested elements.
     */
    public CredentialsElement createCredentials() {
        // check if there are duplicates of the "credentials" element
        if (credentials != null) {
            throw new BuildException("Only one nested credentials element is allowed");
        }
        credentials = new CredentialsElement();
        return credentials;
    }

    /**
     * <p>
     * Returns the nested "credentials" element. The "credentials" element specifies the authentication properties for
     * accessing the server.
     * </p>
     *
     * @return the nested "credentials" element, or null if none is specified.
     */
    public CredentialsElement getCredentials() {
        return credentials;
    }
}
