/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DynamicElement;

import java.lang.reflect.InvocationTargetException;


/**
 * <p>
 * This class is used to specify the actual server to be used while running tests. The server must be of type Server.
 * </p>
 *
 * <p>
 * In the build file, this class represents the child element under the ServerAppTestTask. The name of the element is
 * the fully qualified class name of the Server implementation, and is supplied to the constructor. The element
 * supports no attributes. It should include one nested Server element (required).
 * </p>
 *
 * <p>
 * This class is not thread-safe. However the methods will be called one by one by the Ant framework according to the
 * life cycle of ant task. So thread-safety is not an issue here.
 * </p>
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public class ServerElement implements DynamicElement {
    /**
     * <p>
     * Represents the name of the element represented by this class. It is the fully qualified class name of the Server
     * implementation. It is set in the constructor and then never changed. It cannot be null or empty string.
     * </p>
     */
    private final String className;

    /**
     * <p>
     * The nested server element, which represents the actual server to be used while running tests. This element is
     * required. It is initially null, and set later by the createDynamicElement() method.
     * </p>
     */
    private Server server = null;

    /**
     * <p>
     * Represents an encapsulation of the information about the ant task. It is initially null, and set later by the
     * setAntTaskInfo() method.
     * </p>
     */
    private AntTaskInfo antTaskInfo = null;

    /**
     * <p>
     * Creates a new instance of this class with the specified className. The className is the name of the element
     * represented by this class. It should be the fully qualified class name of the Server implementation.
     * </p>
     *
     * @param className the fully qualified class name of the Server implementation.
     *
     * @throws IllegalArgumentException if className is null or empty string.
     */
    public ServerElement(String className) {
        if (className == null) {
            throw new IllegalArgumentException("className is null");
        }
        if (className.trim().length() == 0) {
            throw new IllegalArgumentException("className is empty string");
        }
        this.className = className;
    }

    /**
     * <p>
     * Creates a nested element with the specified name. The name specifies the type of the server. The nested element
     * must be of type Server, whose class name is given in the constructor.
     * </p>
     *
     * @param name the name of the nested element.
     *
     * @return the nested element, must be of type Server.
     *
     * @throws IllegalArgumentException if name is null or empty string.
     * @throws BuildException if there are more than one nested elements; or when it fails to create a server, e.g. the
     *         type is not supported by that server.
     */
    public Object createDynamicElement(String name) {
        if (name == null) {
            throw new IllegalArgumentException("name is null");
        }
        if (name.trim().length() == 0) {
            throw new IllegalArgumentException("name is empty string");
        }

        // check if there are duplicates of the server
        if (server != null) {
            throw new BuildException("Only one nested server element is allowed");
        }

        // locate the server class
        Class clazz = null;

        try {
            clazz = Class.forName(className);
        } catch (ClassNotFoundException e) {
            throw new BuildException("Unable to locate the class " + className, e);
        }

        if (!Server.class.isAssignableFrom(clazz)) {
            throw new BuildException("Class " + className + " does not implement Server");
        }

        // prepare the constructor parameters
        Class[] parameterTypes = new Class[] {String.class};
        Object[] parameters = new Object[] {name};

        // instantiate the server class
        try {
            server = (Server) clazz.getConstructor(parameterTypes).newInstance(parameters);
        } catch (NoSuchMethodException e) {
            throw new BuildException("Unable to locate the constructor", e);
        } catch (InstantiationException e) {
            throw new BuildException("Fails to create the instance", e);
        } catch (IllegalAccessException e) {
            throw new BuildException("Fails to create the instance", e);
        } catch (InvocationTargetException e) {
            throw new BuildException("Fails to create the instance", e);
        }

        // pass the ant task information
        server.setAntTaskInfo(antTaskInfo);

        return server;
    }

    /**
     * <p>
     * Returns the actual server to be used while running tests.
     * </p>
     *
     * @return the actual server to be used while running tests, or null if none is specified.
     */
    public Server getServer() {
        return server;
    }

    /**
     * <p>
     * Sets the information about the ant task being run. Note that the argument is checked here since it is not an
     * attribute set by the Ant framework.
     * </p>
     *
     * @param antTaskInfo the information about the ant task.
     *
     * @throws IllegalArgumentException if antTaskInfo is null
     */
    public void setAntTaskInfo(AntTaskInfo antTaskInfo) {
        if (antTaskInfo == null) {
            throw new IllegalArgumentException("antTaskInfo is null");
        }
        this.antTaskInfo = antTaskInfo;
    }

    /**
     * <p>
     * Returns the information about the ant task being run.
     * </p>
     *
     * @return the information about the ant task being run.
     */
    public AntTaskInfo getAntTaskInfo() {
        return antTaskInfo;
    }
}
