/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework.web;

import com.topcoder.testframework.AntTaskInfo;
import com.topcoder.testframework.ApplicationServer;

import org.apache.cactus.integration.ant.container.Container;
import org.apache.cactus.integration.ant.container.ContainerFactory;
import org.apache.cactus.integration.ant.container.ContainerRunner;
import org.apache.cactus.integration.ant.deployment.DeployableFile;
import org.apache.cactus.integration.ant.deployment.EarParser;
import org.apache.cactus.integration.ant.deployment.WarParser;
import org.apache.cactus.integration.ant.util.DefaultAntTaskFactory;

import org.apache.tools.ant.BuildException;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import java.net.MalformedURLException;
import java.net.URL;


/**
 * <p>
 * This class represents the web application server to be used while running tests. It uses Cactus as a back-end. As
 * such, the test cases it executes also use Cactus as a back-end.
 * </p>
 *
 * <p>
 * In the build file, this class represents the child element under the ServerElement. The name of the element is
 * supplied to the constructor. It should be the name of the Cactus container type, which should be one of:
 * <b>jboss3x</b>, <b>orion1x</b>, <b>orion2x</b>, <b>resin2x</b>, <b>resin3x</b>, <b>tomcat3x</b>, <b>tomcat4x</b>,
 * <b>tomcat5x</b>, <b>weblogic7x</b>. The element supports the following attributes, in addition to the attributes
 * supported by the base class:
 *
 * <ul>
 * <li>
 * <b>timeout</b> (optional) - the timeout after which to stop trying to call the container when checking if the
 * container is running (in milliseconds). Must be non-negative. Default is 3 minutes.
 * </li>
 * <li>
 * <b>testurl</b> (optional) - the URL used to check if the server is running. This attribute is required only when
 * neither "warfile" nor "earfile" attributes are specified.
 * </li>
 * </ul>
 *
 * If the "port" attribute is not specified, the default port will depend on the specific Cactus container
 * implementation. Due to the limitation of the Cactus back-end, the "startserver" and "initserver" attributes are
 * ignored. They do not have any effect on server and data initialization. If there is a nested "credentials" element,
 * the credentials information will be ignored as well.
 * </p>
 *
 * <p>
 * <i>Note: it is not recommended to have multiple DefaultWebApplicationServers specified in the task.</i> If this is
 * the case, the value of the "cactus.contextURL" property will be set based on the last server in the list.
 * </p>
 *
 * <p>
 * For those methods called by the Ant framework, such as attribute setters, there is no check for arguments against
 * null. The Ant framework makes sure that the arguments will not be null.
 * </p>
 *
 * <p>
 * This class is not thread-safe. However the methods will be called one by one by the Ant framework according to the
 * life cycle of ant task. So thread-safety is not an issue here.
 * </p>
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public class DefaultWebApplicationServer extends ApplicationServer {
    /**
     * <p>
     * The name of the method in the concrete container class to set the installation directory of the server.
     * </p>
     */
    private static final String SET_DIR_METHOD_NAME = "setDir";

    /**
     * <p>
     * The name of the method in the concrete container class to set the TCP port that the server should listen to.
     * </p>
     */
    private static final String SET_PORT_METHOD_NAME = "setPort";

    /**
     * <p>
     * The name of the method in the concrete container class to set the host name or IP address of the machine running
     * the server.
     * </p>
     */
    private static final String SET_SERVER_METHOD_NAME = "setServer";

    /**
     * <p>
     * The property key specifying the context URL under which the application runs the tests.
     * </p>
     */
    private static final String CONTEXT_URL_PROPERTY_KEY = "cactus.contextURL";

    /**
     * <p>
     * The query string appended to the default test URL to check if the server is running.
     * </p>
     */
    private static final String CACTUS_URL_QUERY_STRING = "?Cactus_Service=RUN_TEST";

    /**
     * <p>
     * The Cactus container representing the server to be used while running tests. It is initialized in the
     * constructor and then never changed. It cannot be null.
     * </p>
     */
    private final Container container;

    /**
     * <p>
     * The Cactus container runner to be used for starting and stopping the server represented by the container
     * variable. It is initialized in the constructor and then never changed. It cannot be null.
     * </p>
     */
    private final ContainerRunner runner;

    /**
     * <p>
     * The "timeout" attribute specifies the timeout after which to stop trying to call the container when checking if
     * the container is running (in milliseconds). This attribute is optional. It is initially set to 3 minutes, and
     * set later by the setTimeout() method.
     * </p>
     */
    private long timeout = 180000;

    /**
     * <p>
     * The "testurl" attribute specifies the URL used to check if the server is running. This attribute is required
     * only when niether "warfile" nor "earfile" attributes are specified. It is initially null, and set later by the
     * setURL() method.
     * </p>
     */
    private URL testURL = null;

    /**
     * <p>
     * Creates a new instance of this class with the specified type. The type is the name of the element represented by
     * this class, such as "tomcat5x".
     * </p>
     *
     * @param type the type of the server.
     *
     * @throws IllegalArgumentException if type is null or empty string.
     * @throws BuildException if the type of the server is unrecognized.
     */
    public DefaultWebApplicationServer(String type) {
        super(type);

        ContainerFactory factory = new ContainerFactory();

        // create the Cactus container and its runner
        container = factory.createContainer(type);
        runner = new ContainerRunner(container);
    }

    /**
     * <p>
     * Starts the server. This method should be called before running the tests.
     * </p>
     *
     * <p>
     * Due to the limitation of the Cactus back-end, the "startserver" attribute is ignored here. The server is started
     * regardless of the "startserver" attribute.
     * </p>
     *
     * @throws BuildException if any required attribute or nested element is missing; or it fails to start up the
     *         server.
     */
    public void startUp() {
        // check for required attribute
        if (getDir() == null) {
            throw new BuildException("The dir attribute must be specified");
        }

        // check for testurl attribute if both warfile and earfile attributes are absent
        if ((getWarFile() == null) && (getEarFile() == null) && (testURL == null)) {
            throw new BuildException(
                "The testurl attribute must be specified when both warfile and earfile attributes are not specified");
        }

        // check if both warfile and earfile attributes are present
        if ((getWarFile() != null) && (getEarFile() != null)) {
            throw new BuildException("The warfile and earfile attributes must not be specified together");
        }

        // pass the ant task information to the container
        AntTaskInfo taskInfo = getAntTaskInfo();

        container.setAntTaskFactory(new DefaultAntTaskFactory(taskInfo.getProject(), taskInfo.getTaskName(),
                taskInfo.getLocation(), taskInfo.getTarget()));

        // parse deployment descriptors for WAR or EAR files
        DeployableFile deployableFile = null;

        if (getWarFile() != null) {
            deployableFile = WarParser.parse(getWarFile());
        } else if (getEarFile() != null) {
            deployableFile = EarParser.parse(getEarFile());
        }
        container.setDeployableFile(deployableFile);

        // set the container parameters, including the dir, server and port
        // we use reflection here since the methods are not part of the Container interface,
        // but should be present in all the supported containers
        invoke(container, SET_DIR_METHOD_NAME, getDir());
        invoke(container, SET_SERVER_METHOD_NAME, getServer());

        if (getPort() != -1) {
            invoke(container, SET_PORT_METHOD_NAME, Integer.TYPE, new Integer(getPort()));
        }

        // determine the context URL to run the tests from, and
        // the test URL to check if the server is running
        String contextURL = null;
        URL url = null;

        if (testURL != null) {
            // use the testURL if specified
            contextURL = testURL.toString();
            url = testURL;
        } else {
            // otherwise compose the URLs from the properties of container and deployable file
            contextURL = container.getBaseURL() + "/" + deployableFile.getTestContext();
            try {
                url = new URL(contextURL + deployableFile.getServletRedirectorMapping() + CACTUS_URL_QUERY_STRING);
            } catch (MalformedURLException e) {
                throw new BuildException("Malformed test URL", e);
            }
        }

        // make sure the context URL property is available on the client side
        System.setProperty(CONTEXT_URL_PROPERTY_KEY, contextURL);

        // set the runner parameters, including the timeout and url
        runner.setTimeout(timeout);
        runner.setURL(url);

        // make sure the container is initialized
        container.init();

        // start the container in another thread
        runner.startUpContainer();
    }

    /**
     * <p>
     * Stops the server. This method should be called after running the tests.
     * </p>
     *
     * <p>
     * Due to the limitation of the Cactus back-end, the "startserver" attribute is ignored here. The server is stopped
     * regardless of the "startserver" attribute.
     * </p>
     */
    public void shutDown() {
        runner.shutDownContainer();
    }

    /**
     * <p>
     * Initializes the server with data. This method should be called before running the tests.
     * </p>
     *
     * <p>
     * Due to the limitation of the Cactus back-end, the initialization of data cannot be separated from startup. The
     * method does nothing here. The "initserver" attribute is ignored.
     * </p>
     */
    public void initData() {
    }

    /**
     * <p>
     * Cleans up the data for the server initialized by the initData() method. This method should be called after
     * running the tests.
     * </p>
     *
     * <p>
     * Due to the limitation of the Cactus back-end, the cleaning up of data cannot be separated from shutdown. The
     * method does nothing here. The "initserver" attribute is ignored.
     * </p>
     */
    public void cleanUpData() {
    }

    /**
     * <p>
     * Sets the value of the "timeout" attribute. The "timeout" attribute specifies the timeout after which to stop
     * trying to call the container when checking if the container is running (in milliseconds).
     * </p>
     *
     * @param timeout the value of the "timeout" attribute.
     *
     * @throws IllegalArgumentException if timeout is negative
     */
    public void setTimeout(long timeout) {
        if (timeout < 0) {
            throw new IllegalArgumentException("timeout must be non-negative");
        }
        this.timeout = timeout;
    }

    /**
     * <p>
     * Returns the value of the "timeout" attribute. The "timeout" attribute specifies the timeout after which to stop
     * trying to call the container when checking if the container is running (in milliseconds).
     * </p>
     *
     * @return the value of the "timeout" attribute.
     */
    public long getTimeout() {
        return timeout;
    }

    /**
     * <p>
     * Sets the value of the "testurl" attribute. The "testurl" attribute specifies the URL used to check if the server
     * is running.
     * </p>
     *
     * @param testURL the value of the "testurl" attribute.
     */
    public void setTestURL(URL testURL) {
        this.testURL = testURL;
    }

    /**
     * <p>
     * Returns the value of the "testurl" attribute. The "testurl" attribute specifies the URL used to check if the
     * server is running.
     * </p>
     *
     * @return the value of the "testurl" attribute, or null if none is specified.
     */
    public URL getTestURL() {
        return testURL;
    }

    /**
     * <p>
     * Invokes the method of the specified name for the container. The single method argument is given. For primitive
     * type, the argument should be converted to the corresponding wrapper class.
     * </p>
     *
     * @param container the container to invoke the method from.
     * @param methodName the name of the method to invoke.
     * @param argument the single method argument.
     *
     * @throws BuildException if any error occurs in invoking the method.
     */
    private void invoke(Container container, String methodName, Object argument) {
        invoke(container, methodName, argument.getClass(), argument);
    }

    /**
     * <p>
     * Invokes the method of the specified name for the container. The single method argument and its type is given.
     * For primitive type, the argument should be converted to the corresponding wrapper class.
     * </p>
     *
     * @param container the container to invoke the method from.
     * @param methodName the name of the method to invoke.
     * @param clazz the type of the method argument.
     * @param argument the single method argument.
     *
     * @throws BuildException if any error occurs in invoking the method.
     */
    private void invoke(Container container, String methodName, Class clazz, Object argument) {
        // prepare the method argument
        Class[] parameterTypes = new Class[] {clazz};
        Object[] parameters = new Object[] {argument};

        // locate the method
        Method method = null;

        try {
            method = container.getClass().getMethod(methodName, parameterTypes);
        } catch (NoSuchMethodException e) {
            throw new BuildException("Unable to locate the method " + methodName, e);
        }

        // invoke the method with argument
        try {
            method.invoke(container, parameters);
        } catch (IllegalAccessException e) {
            throw new BuildException("Fails to invoke the method", e);
        } catch (InvocationTargetException e) {
            throw new BuildException("Fails to invoke the method", e);
        }
    }
}
