/*
 * Copyright (C) 2006 TopCoder Inc., All Rights Reserved.
 */
package com.topcoder.testframework.web;

import org.apache.cactus.integration.ant.CactifyEarTask;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.FileSet;

import java.io.File;


/**
 * <p>
 * This class represents an Ant task that injects elements necessary to run the tests into an existing EAR (Enterprise
 * Archive) file. It is used to prepare the EAR file for the DefaultWebApplicationServer. For now, it is a wrapper for
 * CactifyEarTask (in Cactus), but future implementations may change.
 * </p>
 *
 * <p>
 * In the build file, this class represents the root task element. The name of the element is defined by the taskdef
 * core task. The element supports the following attributes:
 *
 * <ul>
 * <li>
 * <b>srcfile</b> (required) - the source ear file that is going to be cactified.
 * </li>
 * <li>
 * <b>destfile</b> (required) - the destination ear file to create.
 * </li>
 * </ul>
 *
 * Under this task element, the <b>fileset</b> nested element is used to specify jars or wars to be added to the ear.
 * Classes and libs are always added as jars.
 * </p>
 *
 * <p>
 * Here is a snippet of the build file, showing the structure of the PrepareTestEarTask:
 * <pre>
 * &lt;taskdef name="preparetestear" classname="com.topcoder.testframework.PrepareTestEarTask"/&gt;
 *
 * &lt;!-- Prepare existing enterprise archive for testing --&gt;
 * &lt;preparetestear srcfile="${target.dir}/${project.name.file}.ear" destfile="${test.dir}/test.ear"&gt;
 *   &lt;fileset dir="${src.test.dir}"&gt;
 *   &lt;/fileset&gt;
 * &lt;/preparetestear&gt;
 * </pre>
 * </p>
 *
 * <p>
 * For those methods called by the Ant framework, such as attribute setters, there is no check for arguments against
 * null. The Ant framework makes sure that the arguments will not be null.
 * </p>
 *
 * <p>
 * This class is not thread-safe. However the methods will be called one by one by the Ant framework according to the
 * life cycle of ant task. So thread-safety is not an issue here.
 * </p>
 *
 * @author real_vg, TCSDEVELOPER
 * @version 1.0
 */
public class PrepareTestEarTask extends Task {
    /**
     * <p>
     * The wrapped CactifyEarTask instance to cactify the ear file. It is initialized in the constructor and never
     * changed. Cannot be null.
     * </p>
     */
    private final CactifyEarTask cactifyEarTask;

    /**
     * <p>
     * Default constructor.
     * </p>
     */
    public PrepareTestEarTask() {
        super();
        cactifyEarTask = new CactifyEarTask();
    }

    /**
     * <p>
     * Sets the value of "srcfile" attribute. The "srcfile" attribute specifies the source ear file that is going to be
     * cactified.
     * </p>
     *
     * @param srcFile the value of "srcfile" attribute.
     */
    public void setSrcFile(File srcFile) {
        cactifyEarTask.setSrcFile(srcFile);
    }

    /**
     * <p>
     * Sets the value of "destfile" attribute. The "destfile" attribute specifies the destination ear file to create.
     * </p>
     *
     * @param destFile the value of "destfile" attribute.
     */
    public void setDestFile(File destFile) {
        cactifyEarTask.setDestFile(destFile);
    }

    /**
     * <p>
     * Adds a set of files, possibly jars and wars to be added to the ear. These may include other classes and libs.
     * </p>
     *
     * @param fileSet the files to add.
     *
     * @throws IllegalArgumentException if fileSet is null.
     */
    public void addFileset(FileSet fileSet) {
        if (fileSet == null) {
            throw new IllegalArgumentException("fileSet is null");
        }
        cactifyEarTask.addFileset(fileSet);
    }

    /**
     * <p>
     * Executes the task. Cactifies the source ear file into destination ear file. If "srcfile" or "destfile" attribute
     * is not specified, a BuildException will be thrown.
     * </p>
     *
     * @throws BuildException if it fails to prepare the ear file.
     */
    public void execute() {
        cactifyEarTask.setProject(getProject());

        try {
            cactifyEarTask.execute();
        } catch (BuildException e) {
            // for BuildException, simply re-throw it
            throw e;
        } catch (Exception e) {
            // for other exceptions, wrap it with BuildException
            throw new BuildException("Fails to prepare ear file", e);
        }
    }
}
